import random, sys
import numpy as np
import pandas as pd
import networkx as nx
from scipy import sparse
from scipy.stats import rankdata
from collections import defaultdict

def give_comm_polarity(comm, polarities):
   global _user_polarity
   comm_neighbors = defaultdict(int)
   for mate in comm:
      polarity = polarities.pop()
      G.nodes[mate]["polarity"] = polarity
      G.nodes[mate]["polarity_set"] = True
      polarity_user[polarity] = mate
      for neigh in set(G[mate].keys()) - G.nodes[mate]["community"]:
         if not G.nodes[neigh]["polarity_set"]:
            comm_neighbors[frozenset(G.nodes[neigh]["community"])] += 1
   for comm2 in sorted(comm_neighbors, key = comm_neighbors.get, reverse = True):
      if not G.nodes[next(iter(comm2))]["polarity_set"]:
         polarities2 = _user_polarity[:len(comm2)]
         _user_polarity = _user_polarity[len(comm2):]
         give_comm_polarity(comm2, polarities2)

# Generate powerlaw popularity distribution for news sources
sys.stderr.write("Generating sources...\n")

source_popularity = pd.read_csv("popularity_distribution_fb.csv", header = None, names = ("popularity",))["popularity"].values.astype(int)

# Generate a normal polarity distribution with avg = 0, and min/max -1/1
source_polarity = np.random.normal(size = source_popularity.shape[0])
source_polarity /= np.max(np.abs(source_polarity))
np.save("source_polarity", source_polarity)

# Generate the polarity of the audience, with avg = 0, and min/max -1/1
sys.stderr.write("Generating social network...\n")
number_users = int(max(source_popularity) * 2.5)
user_polarity = np.random.normal(size = number_users)
user_polarity /= np.max(np.abs(user_polarity))

# Make the social network, an LFR benchmark
while True:
   try:
      G = nx.generators.community.LFR_benchmark_graph(number_users, 3, 1.2, 0.06, min_community = 18, average_degree = 8, max_degree = 100)
   except nx.exception.ExceededMaxIterations:
      continue
   else:
      break

G.remove_edges_from(nx.selfloop_edges(G))
nx.set_node_attributes(G, {n: 0 for n in G.nodes}, name = "polarity")
nx.set_node_attributes(G, {n: False for n in G.nodes}, name = "polarity_set")
#nx.write_edgelist(G, "social_network", data = False)
sparse.save_npz("social_network", nx.to_scipy_sparse_matrix(G))

# Assign user to polarity
_user_polarity = sorted(user_polarity.copy())
polarity_user = {}
for user in G.nodes:
   if not G.nodes[user]["polarity_set"]:
      polarities = _user_polarity[:len(G.nodes[user]["community"])]
      _user_polarity = _user_polarity[len(G.nodes[user]["community"]):]
      give_comm_polarity(G.nodes[user]["community"], polarities)

user_polarity = np.array([G.nodes[n]["polarity"] for n in G.nodes])
np.save("user_polarity", user_polarity)

sys.stderr.write("Generating audience network...\n")
# Assign users to news sources. Each news source gets its popularity worth of users,
# and we pick them so that they're the closest possible in polarity
user_nsources = defaultdict(int)
rows = []
cols = []
vals = []
for i in range(len(source_popularity)):
   polarity_differences = np.abs(user_polarity - source_polarity[i])
   idx = rankdata(polarity_differences)
   audience_polarities = user_polarity[idx <= source_popularity[i]]
   for audience_polarity in audience_polarities:
      rows.append(i)
      cols.append(polarity_user[audience_polarity])
      vals.append(1)
      user_nsources[polarity_user[audience_polarity]] += 1

audience_network = sparse.coo_matrix((vals, (rows, cols)), shape = (source_polarity.shape[0], number_users))
sparse.save_npz("audience_network", audience_network)
