import glob, sys, ipdb
import numpy as np
import pandas as pd
import networkx as nx
from scipy import sparse

def run(shareability, flaggability, greediness, skepticism):
   params = f"{shareability}_{flaggability}_{greediness}_{skepticism}"
   runs = glob.glob(f"results/{params}/sources_*.csv")
   if len(runs) > 0:
      df = []
      for hist in runs:
         df.append(pd.read_csv(hist, sep = "\t"))
      df = pd.concat(df)
      df.to_csv(f"processed_results/sources/{params}.csv", index = False, sep = "\t")
      runs = glob.glob(f"results/{params}/users_*.csv")
      df = []
      for hist in runs:
         df.append(pd.read_csv(hist, sep = "\t"))
      df = pd.concat(df)
      df["user"] = range(df.shape[0])
      df.to_csv(f"processed_results/users/{params}.csv", index = False, sep = "\t")
      runs = glob.glob(f"results/{params}/*.log")
      dfs = []
      for hist in runs:
         try:
            df = pd.read_csv(hist, sep = "\t")
            usrpol_final = df.iloc[-1]["usrpol"]
            srcpol_final = df.iloc[-1]["srcpol"]
            df = df.set_index("step").reindex(range(1, 101))
            df[["usrchng", "srcchng"]] = df[["usrchng", "srcchng"]].fillna(0.0)
            df["usrpol"] = df["usrpol"].fillna(usrpol_final)
            df["srcpol"] = df["srcpol"].fillna(srcpol_final)
            dfs.append(df)
         except pd.errors.EmptyDataError:
            continue
      df = pd.concat(dfs)
      df = df.groupby(by = "step").mean().reset_index()
      df.to_csv(f"processed_results/runs/{params}.csv", index = False, sep = "\t")
      social = sparse.load_npz("synth_data/run1/social_network.npz").tocsr()
      G = nx.from_scipy_sparse_matrix(social)
      nx.write_edgelist(G, f"processed_results/social/start", delimiter = "\t", data = False)
      social2 = sparse.load_npz(f"results/{params}/social_1.npz").tocsr()
      G = nx.from_scipy_sparse_matrix(social2)
      nx.write_edgelist(G, f"processed_results/social/end_{params}", delimiter = "\t", data = False)

shareability = float(sys.argv[1])
flaggability = float(sys.argv[2])
greediness = float(sys.argv[3])
skepticism = float(sys.argv[4])
run(shareability, flaggability, greediness, skepticism)
