import sys
import pandas as pd
from scipy.stats import wasserstein_distance

rho = sys.argv[1]
phi = sys.argv[2]
gamma = sys.argv[3]
sigma = sys.argv[4]

# We need to renormalize the mediabiasfactcheck scores to be actually between -1 and +1, then bin them and count how many users are in each bin
real = pd.read_csv("abortion_user_scores.csv", header = None, names = ("user", "pol"))
real.loc[real["pol"] < 0, "pol"] /= 0.7
real.loc[real["pol"] > 0, "pol"] /= 0.8
real.loc[real["pol"] < -1, "pol"] =-1
real.loc[real["pol"] > 1, "pol"] = 1

real["pol"] = real["pol"].round(1)
real = real.groupby(by = "pol").size().reset_index().rename(columns = {0: "count"})

# We need to ensure symmetry, since the model doesn't take into account relative popularity of each side of the polarity spectrum
left_max = real[real["pol"] < 0]["count"].max()
right_max = real[real["pol"] > 0]["count"].max()

right_left_ratio = right_max / left_max
real.loc[real["pol"] > 0, "count"] /= right_left_ratio
real["count"] /= real["count"].sum()

# We bin and count the user polarities in the model in the same way we did for the real data
df = pd.read_csv(f"../04_user_mobile_model/processed_results/users/{rho}_{phi}_{gamma}_{sigma}.csv", sep = "\t")
df["final_polarity"] = df["final_polarity"].round(1)
df = df.groupby(by = "final_polarity").size().reset_index().rename(columns = {0: "count"})
df["count"] /= df["count"].sum()

df = df.merge(real, left_on = "final_polarity", right_on = "pol", how = "outer").fillna(0.0)

pearson_r, pearson_p = pearsonr(df["count_x"], df["count_y"])
print(f"{rho}\t{phi}\t{gamma}\t{sigma}\t{pearson_r}\t{pearson_p}")
