import pandas as pd
import networkx as nx
from scipy import sparse

users = []
sources = []
for step in (0, 16, 32, 48, 64, 80, 96):
   social = sparse.load_npz(f"steps/social_{step}.npz").tocsr()
   G = nx.from_scipy_sparse_matrix(social)
   nx.write_edgelist(G, f"social_step{step}", delimiter = "\t", data = False)
   df = pd.read_csv(f"steps/users_{step}.csv", sep = "\t")
   df["step"] = step
   users.append(df)
   df = pd.read_csv(f"steps/sources_{step}.csv", sep = "\t")
   df["step"] = step
   sources.append(df)

users = pd.pivot_table(data = pd.concat(users).reset_index(), index = "index", columns = "step", values = "polarity").reset_index().rename(columns = {"index": "user"})
users.to_csv("user_polarity_bystep.csv", sep = "\t", index = False)

sources = pd.pivot_table(data = pd.concat(sources).reset_index(), index = "index", columns = "step", values = "polarity").reset_index().rename(columns = {"index": "source"})
sources.to_csv("source_polarity_bystep.csv", sep = "\t", index = False)
